<?php
namespace PME\Controllers\Custo;

use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;

class ConsumoPlanejadoRealizadoController extends \Zeedhi\Framework\Controller\Report {

    protected $entityManager;
    protected $birtStrategy;
    protected $environment;
    protected $newCode;
    protected $connection;
    protected $lista;

    public function __construct(ReportService $reportService, $entityManager, $birtStrategy, $environment, $newCode, $connection, $lista) {
        $this->entityManager = $entityManager;
        $this->birtStrategy  = $birtStrategy;
        $this->environment   = $environment;
        $this->newCode       = $newCode;
        $this->connection    = $connection;
        $this->lista = $lista;
        
        parent::__construct($reportService);
    }

    protected $reportMapping = array (
      'PME01800' => 
      array (
        'strategy' => 'BIRT',
        'parameterMapping' => 
        array (
          "TAMACNIVEL"   => "P_NRTAMACNIVEL",
          "TAMCODIGO"    => "P_NRTAMCODIGO",
          "NIVEL"        => "P_NIVEL",
          "CDFILCENTPME" => "P_CDFILCENTPME",
          "NRCONSPLAN"   => "P_NRCONSPLAN",
          "LISTASERV"    => "P_CDSERVFILI", 
          "PRODINICIAL"  => "P_CDPRODINIC",
          "PRODFINAL"    => "P_CDPRODFINA",
          "DTDE"         => "P_DTINICIAL",
          "DTATE"        => "P_DTFINAL",
          "ORDENA"       => "P_ORDENA",
          "DIFQTD"       => "IDDIFERENCA",
          "CDPICTPROD"   => "P_CDPICTPROD",
          "AGRUPMASK"    => "P_MASCAGRUP",
          "NRORG"        => "P_NRORG"
        )
      )
    );
    
    public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response) {
      $rows = $request->getRow();
      $nrorg = $this->environment->getNrOrgTrab();
      
      $rows['NRORG'] = $nrorg;
      $rows['NRCONSPLAN'] = $this->newCode->getCodeByTableName('CONSPLAN'.$rows['CDFILCENTPME'], $nrorg);
      
      $unidadeServico = [];
      
      foreach($rows['SERVICO'] as $row){
        $concatUnidServ = $row['FILISERV'];
        array_push($unidadeServico, $concatUnidServ);
      }
      
      if (empty($rows['SERVICO'])) {
        $unidServ = $this->getUnidServ($rows);
        $rows['LISTASERV'] = $this->lista->geraLista($unidServ, $nrorg);
      } else $rows['LISTASERV'] = $this->lista->geraLista(join("_", $unidadeServico), $nrorg);
      
      if($rows['ORDENA'] == 0) $rows['TAMCODIGO'] = 10;
      else $rows['TAMCODIGO'] = 0;
      
      $cdprodIni = $rows['PRODINICIAL']['CDARVPROD'] ? $rows['PRODINICIAL']['CDARVPROD'] : 0;
      if(empty($rows['PRODFINAL']['CDARVPROD'])) $cdprodFin = 'zzzzzzzzzzz';
      else $cdprodFin = str_pad($rows['PRODFINAL']['CDARVPROD'], 10, 'z');

      $rows['PRODINICIAL'] = $cdprodIni;
      $rows['PRODFINAL'] = $cdprodFin;
      
      $this->procedure($rows);
      
      $this->birtStrategy->setReportLocale($this->environment->getLanguage());
      $this->birtStrategy->setReportFormat(strtolower($rows['reportFormat']));
    }
    
    public function procedure($rows){
      $nrorg = $this->environment->getNrOrgTrab();
      $cdoperador = $this->environment->getCdOperador(); 
      
      $servicos = [];
      foreach($rows['SERVICO'] as $row){
        $concatServ = $row['CDSERVFILI'];
        array_push($servicos, $concatServ);
      }
      
      if(empty($rows['PRODINICIAL'])) $cdprodIni = '0';
      else $cdprodIni = $rows['PRODINICIAL'];
      
      if(empty($rows['PRODFINAL'])) $cdprodFin = 'ZZZZZZZZZ';
      else $cdprodFin = str_pad($rows['PRODFINAL'], 10, 'Z');
      
      if ($rows['GRUPFILI'] == NULL) {
        if (empty($rows['UNIDADE'])) {
          $unidade = null;
          $rows['UNIDADE'] = $this->getUnidades($rows);
        } else {
          $rows['UNIDADE'] = is_array($rows['UNIDADE']) ? join("_", $rows['UNIDADE']) : $rows['UNIDADE'];
          $unidade = $rows['UNIDADE'];
        }
      } else {
        if (empty($rows['UNIDADE'])) $rows['UNIDADE'] = $rows['UNIDADE'] = $this->getUnidadesGrupoServ($rows, $nrorg);  
        else $rows['UNIDADE'] = is_array($rows['UNIDADE']) ? join("_", $rows['UNIDADE']) : $rows['UNIDADE'];
        $unidade = $rows['UNIDADE'];
      }
      
      if(empty($rows['SERVICO'])) {
        $rows['SERVICO'] = $this->getServicos($rows);
        $servProce = null;
      } else {
        $rows['SERVICO'] = is_array($rows['SERVICO']) ? join("_", $servicos) : $rows['SERVICO'];
        $servProce = $rows['SERVICO'];
      }
      
      $connection = $this->entityManager->getConnection();
      $procedureCons = new StoredProcedure($connection, 'CONSUMO_PLANEJADO_PME');

      $procedureCons->addParam(new Param('P_CDFILCENTPME', Param::PARAM_INPUT, $rows['CDFILCENTPME'], Param::PARAM_TYPE_STR));
      $procedureCons->addParam(new Param('P_CDTIPOCARD',   Param::PARAM_INPUT, NULL,                  Param::PARAM_TYPE_STR));
      $procedureCons->addParam(new Param('P_CDFILIAL',     Param::PARAM_INPUT, $unidade,      Param::PARAM_TYPE_STR));
      $procedureCons->addParam(new Param('P_CDSERVFILI',   Param::PARAM_INPUT, $servProce,      Param::PARAM_TYPE_STR));
      $procedureCons->addParam(new Param('P_DTINICIAL',    Param::PARAM_INPUT, $rows['DTDE'],         Param::PARAM_TYPE_STR));
      $procedureCons->addParam(new Param('P_DTFINAL',      Param::PARAM_INPUT, $rows['DTATE'],        Param::PARAM_TYPE_STR));
      $procedureCons->addParam(new Param('P_IDARQUIVO',    Param::PARAM_INPUT, 'A',                   Param::PARAM_TYPE_STR));
      $procedureCons->addParam(new Param('P_IDPRODUTOS',   Param::PARAM_INPUT, 'CFP',                 Param::PARAM_TYPE_STR));
      $procedureCons->addParam(new Param('P_IDSEPPRATO',   Param::PARAM_INPUT, 'N',                   Param::PARAM_TYPE_STR));
      $procedureCons->addParam(new Param('P_PEACRESCIMO',  Param::PARAM_INPUT, '0',                   Param::PARAM_TYPE_STR));
      $procedureCons->addParam(new Param('P_IDARREDOND',   Param::PARAM_INPUT, 'N',                   Param::PARAM_TYPE_STR));
      $procedureCons->addParam(new Param('P_NRCONSPLAN',   Param::PARAM_INPUT, $rows['NRCONSPLAN'],   Param::PARAM_TYPE_STR));
      $procedureCons->addParam(new Param('P_CDPRODUTOINI', Param::PARAM_INPUT, $cdprodIni,                   Param::PARAM_TYPE_STR));
      $procedureCons->addParam(new Param('P_CDPRODUTOFIN', Param::PARAM_INPUT, $cdprodFin,           Param::PARAM_TYPE_STR));
      $procedureCons->addParam(new Param('P_CDOPERADOR',   Param::PARAM_INPUT, $cdoperador,           Param::PARAM_TYPE_STR));
      $procedureCons->execute();
    }
    
    public function getUnidades($rows){
      $cdoperador = $this->environment->getCdOperador(); 
      $unidades = [];
      $query = "SELECT DISTINCT FI.CDFILIAL, FI.NMFILIAL, FI.NRORG FROM FILIAL FI, FILIOPER FO WHERE (FI.IDFILCENTPME = 'N') AND (FI.CDFILCENTPME = :CDFILCENTPME) AND (FI.CDFILIAL = FO.CDFILIAL) AND (INSTR(:CDOPERADOR, FO.CDOPERADOR) <> 0 OR :CDOPERADOR IS NULL) ORDER BY FI.CDFILIAL";
      $params = array(
        "CDFILCENTPME" => $rows["CDFILCENTPME"],
        "CDOPERADOR"   => $cdoperador
      );
      $stmt = $this->connection->prepare($query);
		  $stmt->execute($params);
      $result = $stmt->fetchAll();

      foreach($result as $unid){
        array_push($unidades, $unid['CDFILIAL']);
      }
      $resultado = join("_", $unidades);
      
      return $resultado;
    }
    
    public function getUnidadesGrupoServ($rows, $nrorg){
      $cdoperador = $this->environment->getCdOperador(); 
      $unidGrupo = [];
      $query = "SELECT DISTINCT F.CDFILIAL, F.NMFILIAL, F.CDFILIAL || ' - ' || F.NMFILIAL AS DESCRICAO, F.NRORG, F.CDFILCENTPME FROM FILIAL F LEFT JOIN FILGRUPFILI FL ON ( FL.CDGRUPFILI =:CDGRUPFILI OR :CDGRUPFILI = 'T' ) AND FL.NRORG = F.NRORG AND FL.CDFILIAL = F.CDFILIAL WHERE ( FL.CDGRUPFILI IS NOT NULL OR :CDGRUPFILI = 'T' ) AND F.CDFILCENTPME =:CDFILCENTPME AND F.NRORG =:NRORG ORDER BY CDFILIAL";
      $params = array(
        "NRORG"        => $nrorg,
        "CDFILCENTPME" => $rows["CDFILCENTPME"],
        "CDGRUPFILI"   => $rows['GRUPFILI']
      );
      $stmt = $this->connection->prepare($query);
		  $stmt->execute($params);
      $result = $stmt->fetchAll();

      foreach($result as $unid){
        array_push($unidGrupo, $unid['CDFILIAL']);
      }
      $resultado = join("_", $unidGrupo);
      return $resultado;
    }
    
    public function getServicos($rows){
      $nrorg = $this->environment->getNrOrgTrab();
      $servicos = [];
      
      if (empty($rows['GRUPOSERVICO'])) $rows['GRUPOSERVICO'] = 'T';
      if (empty($rows['CDGRUPFILI'])) $rows['CDGRUPFILI'] = 'T';
      
      $query = "SELECT DISTINCT S.CDSERVFILI, S.NMSERVICO, S.NRORG FROM SERVICO      S, FILGRUPFILI  F, FILIAL       FI WHERE ( S.CDFILIAL = F.CDFILIAL (+) ) AND ( FI.CDFILIAL = S.CDFILIAL ) AND ( ( INSTR(:CDFILIAL, S.CDFILIAL) <> 0 ) OR ( :CDFILIAL = 'T' ) ) AND ( ( INSTR(:CDGRUPSERV, S.CDGRUPSERV) <> 0 ) OR ( :CDGRUPSERV = 'T' ) ) AND ( ( INSTR(:CDGRUPFILI, F.CDGRUPFILI) <> 0 ) OR ( :CDGRUPFILI = 'T' ) ) AND ( FI.CDFILCENTPME = :CDFILCENTPME ) ORDER BY S.CDSERVFILI";
      $params = array(
        "CDFILCENTPME" => $rows["CDFILCENTPME"],
        "CDGRUPFILI"   => $rows['CDGRUPFILI'],
        "CDFILIAL"     => $rows['UNIDADE'],
        "CDGRUPSERV"   => $rows['GRUPOSERVICO']
      );
      $stmt = $this->connection->prepare($query);
		  $stmt->execute($params);
      $result = $stmt->fetchAll();
      
      foreach($result as $row){
        array_push($servicos, $row['CDSERVFILI']);
      }
      $resultadoServ = join("_", $servicos);
      return $resultadoServ;
    }
    
    public function getUnidServ($rows){
      $nrorg = $this->environment->getNrOrgTrab();
      $unidServ = [];
      
      if ($rows['GRUPFILI'] == NULL) {
        if (empty($rows['UNIDADE'])) $rows['UNIDADE'] = $this->getUnidades($rows);
        else  $rows['UNIDADE'] = is_array($rows['UNIDADE']) ? join("_", $rows['UNIDADE']) : $rows['UNIDADE'];
      } else if ($rows['GRUPFILI'] !== NULL) {
        if (empty($rows['UNIDADE'])) $rows['UNIDADE'] = $this->getUnidadesGrupoServ($rows, $nrorg);  
        else $rows['UNIDADE'] = join("_", $rows['UNIDADE']);
      }
      if (empty($rows['GRUPOSERVICO'])) $rows['GRUPOSERVICO'] = 'T';
      if (empty($rows['CDGRUPFILI'])) $rows['CDGRUPFILI'] = 'T';
      
      $query = "SELECT DISTINCT S.CDFILIAL, S.CDSERVFILI, S.NMSERVICO, ( S.CDFILIAL || S.CDSERVFILI ) AS FILISERV, S.NRORG FROM SERVICO      S, FILGRUPFILI  F, FILIAL       FI WHERE ( S.CDFILIAL = F.CDFILIAL (+) ) AND ( FI.CDFILIAL = S.CDFILIAL ) AND ( ( INSTR(:CDFILIAL, S.CDFILIAL) <> 0 ) OR ( :CDFILIAL = 'T' ) ) AND ( ( INSTR(:CDGRUPSERV, S.CDGRUPSERV) <> 0 ) OR ( :CDGRUPSERV = 'T' ) ) AND ( ( INSTR(:CDGRUPFILI, F.CDGRUPFILI) <> 0 ) OR ( :CDGRUPFILI = 'T' ) ) AND ( FI.CDFILCENTPME = :CDFILCENTPME ) ORDER BY S.CDFILIAL, S.CDSERVFILI";
      $params = array(
        "CDFILCENTPME" => $rows["CDFILCENTPME"],
        "CDGRUPFILI"   => $rows['CDGRUPFILI'],
        "CDFILIAL"     => $rows['UNIDADE'],
        "CDGRUPSERV"   => $rows['GRUPOSERVICO']
      );
      $stmt = $this->connection->prepare($query);
		  $stmt->execute($params);
      $result = $stmt->fetchAll();
      
      foreach($result as $row){
        array_push($unidServ, $row['FILISERV']);
      }
      $resultadoUnidServ = join("_", $unidServ);
      
      return $resultadoUnidServ;
    }
    
}