<?php
namespace PME\Controllers\Custo;

use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;

class DesvioConsumoController extends \Zeedhi\Framework\Controller\Report {
    protected $birtStrategy;
    protected $environment;
    protected $newCode;
    protected $lista;
    protected $ConsumoPlanejadoRealizadoController;

    public function __construct(ReportService $reportService, $birtStrategy, $environment, $newCode, $lista, $ConsumoPlanejadoRealizadoController) {
        $this->birtStrategy  = $birtStrategy;
        $this->environment   = $environment;
        $this->newCode       = $newCode;
        $this->lista         = $lista;
        $this->ConsumoPlanejadoRealizadoController = $ConsumoPlanejadoRealizadoController;
        
        parent::__construct($reportService);
    }

    protected $reportMapping = array (
      'PME01600' => 
      array (
        'strategy' => 'BIRT',
        'parameterMapping' => 
        array (
          "CDTIPOCUSTO"   => "P_NRTIPOCUSTO",
          "TAMACNIVEL"    => "P_NRTAMACNIVEL",
          "ORDENA"        => "P_NRTAMCODIGO",
          "CDFILCENTPME"  => "P_CDFILCENTPME",
          "NRCONSPLAN"    => "P_NRCONSPLAN",
          "LISTASERV"     => "P_CDSERVFILI", 
          "PRODINICIAL"   => "P_CDPRODINIC",
          "PRODFINAL"     => "P_CDPRODFINA",
          "ESTOCAVEIS"    => "P_IDESTOCAVEIS",
          "DTMESINIC"     => "P_DTMESINIC",
          "DTMESFINA"     => "P_DTMESFINA",
          "NAOESTOCAVEIS" => "P_IDNESTOCAVEIS",
          "DTDE"          => "P_DTINICIAL",
          "DTATE"         => "P_DTFINAL",
          "NMTIPOCUSTO"   => "P_NMTIPOCUSTO",
          "CDPICTPROD"    => "P_CDPICTPROD",
          "DIFQTD"        => "P_IDDIFERENCA",
          "AGRUPMASK"     => "P_MASCAGRUP",
          "UNIDADE"       => "CDFILIAL",
          "NRORG"         => "P_NRORG"
        )
      ),
      'PME01601' => 
      array (
        'strategy' => 'BIRT',
        'parameterMapping' => 
        array (
          "CDTIPOCUSTO"   => "P_NRTIPOCUSTO",
          "TAMACNIVEL"    => "P_NRTAMACNIVEL",
          "ORDENA"        => "P_NRTAMCODIGO",
          "DTDE"          => "P_DTINICIAL",
          "DTATE"         => "P_DTFINAL",
          "LISTASERV"     => "P_CDSERVFILI", 
          "CDFILCENTPME"  => "P_CDFILCENTPME",
          "NRCONSPLAN"    => "P_NRCONSPLAN",
          "PRODINICIAL"   => "P_CDPRODINIC",
          "PRODFINAL"     => "P_CDPRODFINA",
          "ESTOCAVEIS"    => "P_IDESTOCAVEIS",
          "DTMESINIC"     => "P_DTMESINIC",
          "DTMESFINA"     => "P_DTMESFINA",
          "NAOESTOCAVEIS" => "P_IDNESTOCAVEIS",
          "DIFQTD"        => "P_IDDIFERENCA",
          "AGRUPMASK"     => "P_MASCAGRUP",
          "CDPICTPROD"    => "P_CDPICTPROD",
          "NMTIPOCUSTO"   => "P_NMTIPOCUSTO",
          "NRORG"         => "P_NRORG"
        )
      )
    );
    
    public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response) {
      $rows = $request->getRow();
      $nrorg = $this->environment->getNrOrgTrab();
      
      $rows['NRORG'] = $nrorg;
      $rows['NRCONSPLAN'] = $this->newCode->getCodeByTableName('CONSPLAN'.$rows['CDFILCENTPME'], $nrorg);
      
      $unidadeServico = [];
      foreach($rows['SERVICO'] as $row){
        $concatUnidServ = $row['FILISERV'];
        array_push($unidadeServico, $concatUnidServ);
      }
      
      if (empty($rows['SERVICO'])) {
        $unidServ = $this->ConsumoPlanejadoRealizadoController->getUnidServ($rows);
        $rows['LISTASERV'] = $this->lista->geraLista($unidServ, $nrorg);
      } else $rows['LISTASERV'] = $this->lista->geraLista(join("_", $unidadeServico), $nrorg);
      
      if($rows['ORDENA'] == 0) $rows['TAMCODIGO'] = 10;
      else $rows['TAMCODIGO'] = 0;
      
      $cdprodIni = $rows['PRODINICIAL']['CDARVPROD'] ? $rows['PRODINICIAL']['CDARVPROD'] : 0;
      if(empty($rows['PRODFINAL']['CDARVPROD'])) $cdprodFin = 'zzzzzzzzzzz';
      else $cdprodFin = str_pad($rows['PRODFINAL']['CDARVPROD'], 10, 'z');

      $rows['PRODINICIAL'] = $cdprodIni;
      $rows['PRODFINAL'] = $cdprodFin;
      $this->ConsumoPlanejadoRealizadoController->procedure($rows);
      
      $this->birtStrategy->setReportLocale($this->environment->getLanguage());
      $this->birtStrategy->setReportFormat(strtolower($rows['reportFormat']));
    }
}