<?php
namespace PME\Controllers\Servico;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use DateTime;

class ConsPorServicoController extends \PME\Services\TeknisaCrud {
  protected $crudService;
	protected $parameterBag;
	protected $dataSourceName;
	protected $lista;
	protected $environment;
	protected $entityManager;
	protected $connection;
	protected $ConsumoPlanejadoRealizadoController;

  public function __construct($crudService, $parameterBag, $lista, $environment, $entityManager, $connection, $ConsumoPlanejadoRealizadoController) {
      parent::__construct($crudService, $parameterBag);
      $this->lista         = $lista;
      $this->environment   = $environment;
      $this->entityManager = $entityManager;
      $this->connection    = $connection;
      $this->ConsumoPlanejadoRealizadoController = $ConsumoPlanejadoRealizadoController;
	}

  public function saveConsPadr(DTO\Request $request, DTO\Response $response) {
    try {
        $rows = (array) $request->getRow();
        $this->connection->beginTransaction();

        $dataSourceName = 'consServPme';
        $this->callSave($dataSourceName, $rows);
        $this->connection->commit();
        $response->addDataSet(new DataSet("columnsToReturn", $this->crudService->getColumnsToReturn()));
    } catch (\Exception $e) {
      $this->connection->rollback();
      $response->setError(new Error($e->getMessage(), 500));
    }
  }

  public function saveSemCons(DTO\Request $request, DTO\Response $response) {
    try {
        $this->connection->beginTransaction();
        $rows = (array) $request->getRow();
        $rowsToSave = [];

        foreach($rows as $row) {
          if ($row['PERCAPITA'] == 'S' && $row['FIXO'] == 'S') $row['IDTPCONSPME'] = 'A';
          else if ($row['PERCAPITA'] == 'S') $row['IDTPCONSPME'] = 'P';
          else $row['IDTPCONSPME'] = 'F';
          
          array_push($rowsToSave, $row);
        }
        $this->deleteConsServ($rowsToSave);

        $dataSourceName = 'diaSemConsPme';
        $this->callSave($dataSourceName, $rowsToSave);
        $this->connection->commit();
        $response->addDataSet(new DataSet("columnsToReturn", $this->crudService->getColumnsToReturn()));
    } catch (\Exception $e) {
      $this->connection->rollback();
      $response->setError(new Error($e->getMessage(), 500));
    }
  }

  private function deleteConsServ($rows) {
    foreach($rows as $row){
      if ($row['IDTPCONSPME'] == 'A') $tipo = "'P','F'";
      else if ($row['IDTPCONSPME'] == 'F') $tipo = "'F'";
      else $tipo = "'P'";
      
      $query = "DELETE CONSSERVPME WHERE CDFILCENTPME = :CDFILCENTPME AND CDSERVFILI = :CDSERVFILI AND CDFILIAL =:CDFILIAL 
                      AND DTCONSERVPME = :DTCONSERVPME AND IDQTDECONPME IN ($tipo)";

      $params = array(
        'CDFILCENTPME'=> $row['CDFILCENTPME'],
        'CDSERVFILI'  => $row['CDSERVFILI'],
        'CDFILIAL'    => $row['CDFILIAL'],
        'DTCONSERVPME' => $row['DTSEMCONSPME'],
      );

      $this->connection->prepare($query)->execute($params);
    }        
  }

  private function callSave($dataSourceName, $rowsToSave){
    $dataSet = new DataSet($dataSourceName, $rowsToSave);
    $request = new DTO\Request\DataSet($dataSet, "", "", "");  
    $rowsToSave = $this->crudService->getRowsToSave($request, $dataSourceName);
    $this->crudService->save($dataSourceName, $rowsToSave);
  }
  
  public function insertListaExport(DTO\Request $request, DTO\Response $response) {
    try {
      $rows = $request->getRow();
      $nrorg = $this->environment->getNrOrgTrab();

      $unidadeServico = [];
      $servicos = [];
      
      foreach($rows['SERVICO_EXPORT'] as $row){
        $unidServ = $row['UNIDSERV'];
        $concatServ = $row['CDSERVFILI'];
        array_push($unidadeServico, $unidServ);
        array_push($servicos, $concatServ);
      }
      
      $rows['NRLISTA'] = $this->lista->geraLista(join("_", $unidadeServico), $nrorg);

      if(empty($rows['SERVICO_EXPORT'])) $rows['CDFILISERV'] = 'T';
      else if(is_array($rows['SERVICO_EXPORT'])) $rows['CDFILISERV'] = join("_", $servicos);
      
      if(empty($rows['UNIDADE_EXPORT'])) $rows['CDFILIAL'] = 'T';
      else if (is_array($rows['UNIDADE_EXPORT'])) $rows['CDFILIAL'] = join("_", $rows['UNIDADE_EXPORT']);
      
      $this->insertListaProdutos($rows);
      
      if($rows['IDQTDECONS'] === 'Fixo') $rows['IDQTDECONS'] = 'F';
      else $rows['IDQTDECONS'] = 'P';
    
      if($rows['SOBREPOR'] === '0') $rows['SOBREPOR'] = NULL;
      else $rows['SOBREPOR'] = 'S';
      
      $arrayConsGrava = $rows['IDTIPOCONSGRAVA'];
      unset($arrayConsGrava[0]);
      
      foreach($arrayConsGrava as $pos){
        $this->procedure($rows, $pos);
      }
    } catch(\Exception $e) {
      $response->setError(new Error($e->getMessage(), 500));
    }
  }
  
  public function insertListaProdutos($rows){
    $nrorg = $this->environment->getNrOrgTrab();

    $arrayProd = [];
    foreach($rows['PRODUTOS'] as $row){
      $produtos = $row['CDPRODUTO'];
      array_push($arrayProd, $produtos);
    }
    
    $concatProd = join("_", $arrayProd);
    $rows['NRLISTAPROD'] = $this->lista->geraLista($concatProd, $nrorg);
  }
  
  private function procedure($rows, $pos){
    $connection = $this->entityManager->getConnection();
    $procedureCons = new StoredProcedure($connection, 'EXPORTA_CONSUMO');
  
    $procedureCons->addParam(new Param('P_CDFILICENTGRAVA', Param::PARAM_INPUT, $rows['CENTRAL'],                     Param::PARAM_TYPE_STR)); 
    $procedureCons->addParam(new Param('P_IDTIPOCONSGRAVA', Param::PARAM_INPUT, $pos,                                 Param::PARAM_TYPE_STR)); 
    $procedureCons->addParam(new Param('P_DTCONSSERVGRAVA', Param::PARAM_INPUT, $rows['DTCONSSERVGRAVA'],             Param::PARAM_TYPE_STR)); 
    $procedureCons->addParam(new Param('P_CDFILICENTORIG',  Param::PARAM_INPUT, $rows['PRODUTOS'][0]['CDFILCENTPME'], Param::PARAM_TYPE_STR)); 
    $procedureCons->addParam(new Param('P_CDFILIORIG',      Param::PARAM_INPUT, $rows['PRODUTOS'][0]['CDFILIAL'],     Param::PARAM_TYPE_STR)); 
    $procedureCons->addParam(new Param('P_CDSERVORIG',      Param::PARAM_INPUT, $rows['PRODUTOS'][0]['CDSERVFILI'],   Param::PARAM_TYPE_STR)); 
    $procedureCons->addParam(new Param('P_IDQTDECONS',      Param::PARAM_INPUT, $rows['IDQTDECONS'],                  Param::PARAM_TYPE_STR)); 
    $procedureCons->addParam(new Param('P_IDTIPOCONS',      Param::PARAM_INPUT, $rows['PRODUTOS'][0]['IDTIPOCONPME'], Param::PARAM_TYPE_STR)); 
    $procedureCons->addParam(new Param('P_DTCONSSERV',      Param::PARAM_INPUT, $rows['PRODUTOS'][0]['DTCONSERVPME'], Param::PARAM_TYPE_STR)); 
    $procedureCons->addParam(new Param('P_NRLISTA',         Param::PARAM_INPUT, $rows['NRLISTAPROD'],                 Param::PARAM_TYPE_STR)); 
    $procedureCons->addParam(new Param('P_NRLISTAFILISERV', Param::PARAM_INPUT, $rows['NRLISTA'],                     Param::PARAM_TYPE_STR)); 
    $procedureCons->addParam(new Param('P_SOBREPOR',        Param::PARAM_INPUT, $rows['SOBREPOR'],                    Param::PARAM_TYPE_STR)); 
    $procedureCons->execute();
  }
  
  public function cadastroDiaConsumo(DTO\Request $request, DTO\Response $response){
    try {
      $rows = $request->getRow();
      $nrorg = $this->environment->getNrOrgTrab();
      
      $query = "SELECT CDFILCENTPME, CDFILIAL, CDSERVFILI, DTSEMCONSPME, IDTPCONSPME, DTULTATUDIAS FROM DIASEMCONSPME WHERE CDFILCENTPME = :P_CDFILCENTPME AND CDFILIAL = :P_CDFILIAL AND CDSERVFILI = :P_CDSERVFILI AND DTSEMCONSPME = :P_DTSCONSSERV";
      
      $dateStart = str_replace("/", "-", $rows['DTDE']);;
      $dateStart = implode('-', array_reverse(explode('/', substr($dateStart, 0, 10)))).substr($dateStart, 10);
      $dateStart = new DateTime($dateStart);
      
      $dateEnd = str_replace("/", "-", $rows['DTATE']);
      $dateEnd = implode('-', array_reverse(explode('/', substr($dateEnd, 0, 10)))).substr($dateEnd, 10);
      $dateEnd = new DateTime($dateEnd);
      
      $dateRange = array();
      while($dateStart <= $dateEnd){
        $dateRange[] = $dateStart->format('d-m-Y');
        $dateStart = $dateStart->modify('+1day');
      }
      
	    if (count($rows['IDTPCONSPME']) === 2) $rows['IDTPCONSPME'] = 'A';
      else {
        if ($rows['IDTPCONSPME'][0] === 'P') $rows['IDTPCONSPME'] = 'P';
        else $rows['IDTPCONSPME'] = 'F';
      }
      
      foreach ($dateRange as $posDia){ 
        $params = array (
          "P_CDFILCENTPME" => $rows["CDFILCENTPME"],
          "P_CDFILIAL"     => $rows["filtro"]["UNIDADE"],
          "P_CDSERVFILI"   => $rows["filtro"]["SERVICO"],
          "P_DTSCONSSERV"  => $posDia 
        );
        
        $stmt = $this->connection->prepare($query);
  			$stmt->execute($params);
  	    $result = $stmt->fetchAll();
  	    
  	    if (empty($result)) $this->insert($rows, $posDia);
  	    else $this->update($rows, $posDia);
  	    
  	    $this->deleteConsServExcecao($rows, $posDia);
      }
    } catch(\Exception $e) {
      $response->setError(new Error($e->getMessage(), 500));
    }
  }
  
  public function insert($rows, $posDia){
    $query = "INSERT INTO DIASEMCONSPME (CDFILCENTPME, CDFILIAL, CDSERVFILI, DTSEMCONSPME, IDTPCONSPME) VALUES (:P_CDFILCENTPME, :P_CDFILIAL, :P_CDSERVFILI, :P_DTCONSSERV, :P_IDTCONSSERV)";
    $params = array (
      "P_CDFILCENTPME" => $rows["CDFILCENTPME"],
      "P_CDFILIAL"     => $rows["filtro"]["UNIDADE"],
      "P_CDSERVFILI"   => $rows["filtro"]["SERVICO"],
      "P_DTCONSSERV"   => $posDia,
      "P_IDTCONSSERV"  => $rows["IDTPCONSPME"] 
    );
    $stmt = $this->connection->prepare($query);
		$stmt->execute($params);
  }
  
  public function update($rows, $posDia){
    $query = "UPDATE DIASEMCONSPME SET IDTPCONSPME = :IDTCONSSERV WHERE CDFILCENTPME = :OLD_CDFILCENTPME AND CDFILIAL = :OLD_CDFILIAL AND CDSERVFILI = :OLD_CDSERVFILI AND DTSEMCONSPME = :OLD_DTSCONSSERV";
    $params = array (
      "IDTCONSSERV"      => $rows["IDTPCONSPME"], 
      "OLD_CDFILCENTPME" => $rows["CDFILCENTPME"],
      "OLD_CDFILIAL"     => $rows["filtro"]["UNIDADE"],
      "OLD_CDSERVFILI"   => $rows["filtro"]["SERVICO"],
      "OLD_DTSCONSSERV"  => $posDia
    );
    $stmt = $this->connection->prepare($query);
		$stmt->execute($params);
  }
  
  public function deleteConsServExcecao($rows, $posDia){
    $query = "DELETE FROM CONSSERVPME WHERE CDFILCENTPME = :P_CDFILCENTPME AND CDFILIAL = :P_CDFILIAL AND CDSERVFILI = :P_CDSERVFILI AND ((IDQTDECONPME = :P_IDQTDECONS) OR (:P_IDQTDECONS = 'A')) AND IDTIPOCONPME = '9' AND DTCONSERVPME = :P_DTCONSSERV";
    $params = array (
      "P_CDFILCENTPME" => $rows["CDFILCENTPME"],
      "P_CDFILIAL"     => $rows["filtro"]["UNIDADE"],
      "P_CDSERVFILI"   => $rows["filtro"]["SERVICO"],
      "P_IDQTDECONS"   => $rows["IDTPCONSPME"], 
      "P_DTCONSSERV"   => $posDia 
    );
    $stmt = $this->connection->prepare($query);
		$stmt->execute($params);
  }
  
  public function buscaUnidade(DTO\Request $request, DTO\Response $response) {
	  $rows = $request->getRow();
    $nrorg = $this->environment->getNrOrgTrab();
  
    if (empty($rows['UNIDADE'])) $rows['UNIDADE'] = $this->ConsumoPlanejadoRealizadoController->getUnidades($rows);
    else  $rows['UNIDADE'] = is_array($rows['UNIDADE']) ? join("_", $rows['UNIDADE']) : $rows['UNIDADE'];
    
	  $row['NRLISTA'] = $this->lista->geraLista($rows['UNIDADE'], $nrorg);
	  $result = $row['NRLISTA'];
	  
	  $response->addDataSet(new DataSet("result", ["data"=> $result]));
	}
	
	public function buscaServico(DTO\Request $request, DTO\Response $response) {
	  $rows = $request->getRow();
    $nrorg = $this->environment->getNrOrgTrab();
    
    $unidadeServico = [];
    foreach($rows['SERVICOSUBS'] as $row){
      $concatUnidServ = $row['CDCHAVE'];
      array_push($unidadeServico, $concatUnidServ);
    }
  
    if (empty($rows['SERVICOSUBS'])) {
      $unidServ = $this->getUnidServLista($rows);
      $rows['LISTASERVFILI'] = $this->lista->geraLista($unidServ, $nrorg);
    } else $rows['LISTASERVFILI'] = $this->lista->geraLista(join("_", $unidadeServico), $nrorg);
    $result = $rows['LISTASERVFILI'];
  
	  $response->addDataSet(new DataSet("result", ["data"=> $result]));
	}
	
	public function validaProdUnid(DTO\Request $request, DTO\Response $response){
	  $rows = $request->getRow();
	  
	  if (empty($rows['UNIDADE'])) $rows['UNIDADE'] = $this->ConsumoPlanejadoRealizadoController->getUnidades($rows);
    else  $rows['UNIDADE'] = is_array($rows['UNIDADE']) ? join("_", $rows['UNIDADE']) : $rows['UNIDADE'];
    
    $query = "SELECT PF.IDCNTRESTOQ, NVL(PF.IDPRODATIV, 'A') AS IDPRODATIV, IDPRODATIVPR FROM PRODFILI  PF, PRODUTO   P WHERE P.CDPRODUTO = :P_CDPRODUTO AND P.IDPRODATIVPR = 'S' AND PF.CDFILIAL (+) = :P_CDFILIAL AND PF.CDPRODUTO (+) = P.CDPRODUTO";
    if (isset($rows['PRODINICIAL'])){
      $params = array(
        "P_CDFILIAL"  => $rows["UNIDADE"],
        "P_CDPRODUTO" => $rows['PRODINICIAL']['CDPRODUTO']
      );
    } else{
      $params = array(
        "P_CDFILIAL"  => $rows["UNIDADE"],
        "P_CDPRODUTO" => $rows['PRODFINAL']['CDPRODUTO']
      );
    }
	
    $stmt = $this->connection->prepare($query);
		$stmt->execute($params);
	  $result = $stmt->fetchAll();
	  $response->addDataSet(new DataSet("result", ["data"=> $result]));
	}
	
	public function insertSubsProd(DTO\Request $request, DTO\Response $response){
	  try {
  	  $parameters = $request->getParameters();
      $rows = $parameters['checkedRows'];
      
  	  $query = "INSERT INTO CONSSERVPME (CDFILCENTPME, CDFILIAL, CDSERVFILI, IDQTDECONPME, IDTIPOCONPME, DTCONSERVPME, CDPRODUTO, QTCONSERVPME) (SELECT CDFILCENTPME, CDFILIAL, CDSERVFILI, IDQTDECONPME, IDTIPOCONPME, DTCONSERVPME, TRIM(LPAD(:P_CDPRODDEST, 10, ' ')), QTCONSERVPME FROM CONSSERVPME WHERE CDFILCENTPME = :P_CDFILCENTPME AND CDFILIAL     = :P_CDFILIAL     AND CDSERVFILI   = :P_CDSERVFILI   AND IDQTDECONPME = :P_IDQTDECONS   AND IDTIPOCONPME = :P_IDTIPOCONS   AND DTCONSERVPME = :P_DTCONSSERV   AND CDPRODUTO    = :P_CDPRODORIG   AND NOT EXISTS (SELECT CDPRODUTO FROM CONSSERVPME WHERE CDFILCENTPME = :P_CDFILCENTPME AND CDFILIAL     = :P_CDFILIAL     AND CDSERVFILI   = :P_CDSERVFILI   AND IDQTDECONPME = :P_IDQTDECONS   AND IDTIPOCONPME = :P_IDTIPOCONS   AND DTCONSERVPME = :P_DTCONSSERV   AND CDPRODUTO  = :P_CDPRODDEST))";
  	  
  	  foreach ($rows as $row){
  	    $params = array (
    	    "P_CDFILIAL"     => $row["CDFILIAL"],
    	    "P_DTCONSSERV"   => $row["DTCONSERV"],
    	    "P_CDFILCENTPME" => $row["CDFILCENTPME"],
    	    "P_CDSERVFILI"   => $row["CDSERVFILI"],
    	    "P_IDTIPOCONS"   => $row["IDTIPOCONPME"],
    	    "P_CDPRODDEST"   => $row["CDPRODDEST"]["CDPRODUTO"],
    	    "P_IDQTDECONS"   => $row["IDQTDECONPME"],
    	    "P_CDPRODORIG"   => $row["CDPRODORIG"]["CDPRODUTO"]
  	    );
  	  }
  	  $stmt = $this->connection->prepare($query);
      $stmt->execute($params);
      
      $this->deleteProdOrigem($rows);
  	  $this->insertLog($rows);
	  } catch(\Exception $e) {
      $response->setError(new Error($e->getMessage(), 500));
    }
	}
	
	public function deleteProdOrigem($rows){
	  $query = "DELETE FROM CONSSERVPME WHERE CDFILCENTPME = :OLD_CDFILCENTPME AND CDSERVFILI = :OLD_CDSERVFILI AND CDFILIAL = :OLD_CDFILIAL AND IDQTDECONPME = :OLD_IDQTDECONPME AND IDTIPOCONPME = :OLD_IDTIPOCONPME AND DTCONSERVPME = :OLD_DTCONSERVPME AND CDPRODUTO = :OLD_CDPRODUTO";
	  foreach($rows as $row){
  	  $params = array (
    	    "OLD_CDFILCENTPME" => $row["CDFILCENTPME"],
    	    "OLD_CDSERVFILI"   => $row["CDSERVFILI"],
    	    "OLD_CDFILIAL"     => $row["CDFILIAL"],
    	    "OLD_IDQTDECONPME" => $row["IDQTDECONPME"],
    	    "OLD_IDTIPOCONPME" => $row["IDTIPOCONPME"],
    	    "OLD_DTCONSERVPME" => $row["DTCONSERV"],
    	    "OLD_CDPRODUTO"    => $row["CDPRODORIG"]["CDPRODUTO"]
  	    );
	  }
	  $this->connection->prepare($query)->execute($params);
	}
	
	public function insertLog($rows){
	  $cdoperador = $this->environment->getCdOperador(); 
    $servUnid = [];
	  
	  $query = "INSERT INTO LOGSUBCONSSER (DTOPERSUB, CDOPERSUB, DSLISFILISUB, DSLISSERVSUB, IDQTDECONSUB, IDTIPOCONSUB, CDPRODORISUB, CDPRODDESSUB) VALUES (:DTOPERSUB, :CDOPERSUB, :DSLISFILISUB, :DSLISSERVSUB, :IDQTDECONSUB, :IDTIPOCONSUB, :CDPRODORISUB, :CDPRODDESSUB)";
	  
	  foreach ($rows as $row) {
	   $array1 = $row['TIPOLOG'];
	   if (is_array($array1[0])) $array1 = $array1[0];
	   $array2 = array("0", "0", "0", "0", "0", "0", "0", "0", "0", "0");
	   for($i = 0; $i < count($array1); $i++){
	     $position = $array1[$i];
	     $array2[$position] = $position;
	   }
	   array_shift($array2);
	   $tipoCons = join("", $array2);
	    
	    if (empty($row['SERVICOSUBS'])){
  	    $servicoConcat = $this->getUnidServ($rows);
	    } else {
	      foreach($row['SERVICOSUBS'] as $unidServ){
  	      array_push($servUnid, $unidServ['CDCHAVE']);
  	    }
  	    $servicoConcat = join("_", $servUnid);
	    }
	    if (empty($row['UNIDADE'])) $unidade = $this->getUnidades($row);
	    else $unidade = is_array($row['UNIDADE']) ? join("_", $row['UNIDADE']) : $row['UNIDADE'];
	    
	    $params = array (
  	    "DTOPERSUB"    => date("d/m/Y H:i:s"),
  	    "CDOPERSUB"    => $cdoperador,
  	    "DSLISFILISUB" => $unidade, 
  	    "DSLISSERVSUB" => $servicoConcat,
  	    "IDQTDECONSUB" => $row["QTLOG"],
  	    "IDTIPOCONSUB" => $tipoCons,
  	    "CDPRODORISUB" => $row["CDPRODORIG"]["CDPRODUTO"],
  	    "CDPRODDESSUB" => $row["CDPRODDEST"]["CDPRODUTO"]
  	  );
	  }
	  
	  $this->connection->prepare($query)->execute($params);
	}
	
	private function getUnidServ($rows){
    $nrorg = $this->environment->getNrOrgTrab();
    $cdoperador = $this->environment->getCdOperador();
    $unidServ = [];
    
    $query = "SELECT DISTINCT  (F.CDFILIAL||S.CDSERVFILI) AS CDCHAVE, F.CDFILIAL, UPPER(FL.NMFILIAL) AS NMFILIAL, S.CDSERVFILI, UPPER(SE.NMSERVICO) AS NMSERVICO, S.NRORG FROM SERVTIPOCARD S, FILITIPOCARD F, SERVICO SE, FILIAL FL, FILIOPER FP, LISTA LF WHERE S.CDFILCENTPME = :P_CDFILCENTPME AND S.CDFILCENTPME = F.CDFILCENTPME  AND S.CDTIPOCARD   = F.CDTIPOCARD    AND SE.CDSERVFILI  = S.CDSERVFILI    AND SE.CDFILIAL    = S.CDFILCENTPME  AND SE.CDFILIAL    = F.CDFILCENTPME  AND FP.CDOPERADOR  = :P_CDOPERADOR   AND LF.NRLISTA     = :P_NRLISTA      AND LF.CDLISTA     = F.CDFILIAL      AND FL.CDFILIAL    = FP.CDFILIAL    AND FL.CDFILIAL    = F.CDFILIAL ORDER BY F.CDFILIAL, S.CDSERVFILI";
    
    foreach ($rows as $row){
      $params = array(
        "P_CDOPERADOR"   => $cdoperador,
        "P_CDFILCENTPME" => $row["CDFILCENTPME"],
        "P_NRLISTA"      => $row['NRLISTA']
      );
    }
    $stmt = $this->connection->prepare($query);
	  $stmt->execute($params);
    $result = $stmt->fetchAll();
    
    foreach($result as $row){
      array_push($unidServ, $row['CDCHAVE']);
    }
    $resultadoUnidServ = join("_", $unidServ);
    
    return $resultadoUnidServ;
  }
  
  private function getUnidServLista($rows){
    $nrorg = $this->environment->getNrOrgTrab();
    $cdoperador = $this->environment->getCdOperador();
    $unidServ = [];
    
    $query = "SELECT DISTINCT  (F.CDFILIAL||S.CDSERVFILI) AS CDCHAVE, F.CDFILIAL, UPPER(FL.NMFILIAL) AS NMFILIAL, S.CDSERVFILI, UPPER(SE.NMSERVICO) AS NMSERVICO, S.NRORG FROM SERVTIPOCARD S, FILITIPOCARD F, SERVICO SE, FILIAL FL, FILIOPER FP, LISTA LF WHERE S.CDFILCENTPME = :P_CDFILCENTPME AND S.CDFILCENTPME = F.CDFILCENTPME  AND S.CDTIPOCARD   = F.CDTIPOCARD    AND SE.CDSERVFILI  = S.CDSERVFILI    AND SE.CDFILIAL    = S.CDFILCENTPME  AND SE.CDFILIAL    = F.CDFILCENTPME  AND FP.CDOPERADOR  = :P_CDOPERADOR   AND LF.NRLISTA     = :P_NRLISTA      AND LF.CDLISTA     = F.CDFILIAL      AND FL.CDFILIAL    = FP.CDFILIAL    AND FL.CDFILIAL    = F.CDFILIAL ORDER BY F.CDFILIAL, S.CDSERVFILI";
    
    foreach ($rows as $row){
      $params = array(
        "P_CDOPERADOR"   => $cdoperador,
        "P_CDFILCENTPME" => $rows["CDFILCENTPME"],
        "P_NRLISTA"      => $rows['NRLISTA']
      );
    }
    $stmt = $this->connection->prepare($query);
	  $stmt->execute($params);
    $result = $stmt->fetchAll();
    
    foreach($result as $row){
      array_push($unidServ, $row['CDCHAVE']);
    }
    $resultadoUnidServ = join("_", $unidServ);
    
    return $resultadoUnidServ;
  }
  
  public function getUnidades($row){
      $cdoperador = $this->environment->getCdOperador(); 
      $unidades = [];
      $query = "SELECT DISTINCT FI.CDFILIAL, FI.NMFILIAL, FI.NRORG FROM FILIAL FI, FILIOPER FO WHERE (FI.IDFILCENTPME = 'N') AND (FI.CDFILCENTPME = :CDFILCENTPME) AND (FI.CDFILIAL = FO.CDFILIAL) AND (INSTR(:CDOPERADOR, FO.CDOPERADOR) <> 0 OR :CDOPERADOR IS NULL) ORDER BY FI.CDFILIAL";
      $params = array(
        "CDFILCENTPME" => $row["CDFILCENTPME"],
        "CDOPERADOR"   => $cdoperador
      );
      $stmt = $this->connection->prepare($query);
		  $stmt->execute($params);
      $result = $stmt->fetchAll();

      foreach($result as $unid){
        array_push($unidades, $unid['CDFILIAL']);
      }
      $resultado = join("_", $unidades);
      
      return $resultado;
    }
}