<?php
namespace PME\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;

class Utilities extends \PME\Services\TeknisaCrud {

    protected $environment;
    protected $connection;
    protected $newCode;


    public function __construct($crudService, $parameterBag, $environment, $connection, $newCode) {
      $this->environment = $environment;
      $this->connection = $connection;
      $this->newCode = $newCode;
        parent::__construct($crudService, $parameterBag);
    }
    
    public function getServicos($central, $unidade = 'T'){
      $conditions = [
        array(
          "columnName" => 'CDFILCENTPME',
          "operator"   => '=',
          "value"      => $central
        ),
        array(
          "columnName" => 'CDFILIAL',
          "operator"   => '=',
          "value"      => $unidade
        ),
        array(
          "columnName" => 'NRORG',
          "operator"   => '=',
          "value"      => $this->environment->getNrOrgTrab()
        )
      ];

      $dataSet = $this->crudService->getFilteredDataSet("servicoUnidCentral", $conditions);
      return $dataSet->getRows();
    }

    public function getUnidades($central){
      $conditions = [
        array(
          "columnName" => 'CDFILCENTPME',
          "operator"   => '=',
          "value"      => $central
        ),
        array(
          "columnName" => 'NRORG',
          "operator"   => '=',
          "value"      => $this->environment->getNrOrgTrab()
        )
      ];
      
      $dataSet = $this->crudService->getFilteredDataSet("agregadaPorCentral", $conditions);  
      return $dataSet->getRows();
    }

    public function getTipoCard($central){
      $conditions = [
        array(
          "columnName" => 'CDFILCENTPME',
          "operator"   => '=',
          "value"      => $central
        ),
        array(
          "columnName" => 'NRPASTATCARD',
          "operator"   => '=',
          "value"      => null
        )
      ];
      
      $dataSet = $this->crudService->getFilteredDataSet("tpCardapioPorCentral", $conditions);  
      return $dataSet->getRows();
    }
    
    public function getDistinctServicos($central, $unidades = null){
      if(!empty($unidades)) $unidades = join("_", $unidades);
      $rows = $this->getServicos($central, $unidades);
      $servicos = [];
      foreach($rows as $item) {
        array_push($servicos, $item['CDSERVFILI']);
      }
      $servicos = array_unique($servicos);
      return $servicos;
    }

    public function getNutrientes ($cdfilial) {
      $sql = "SELECT
                NUT1.NMREDUZIDO AS NMREDUZIDO1,
                NUT1.SGUNIDADE  AS SG1,
                NUT1.CDNUTRIENTE AS CDNUTRIENTE1,
                NUT2.NMREDUZIDO AS NMREDUZIDO2,
                NUT2.SGUNIDADE  AS SG2,
                NUT2.CDNUTRIENTE AS CDNUTRIENTE2,
                NUT3.NMREDUZIDO AS NMREDUZIDO3,
                NUT3.SGUNIDADE  AS SG3,
                NUT3.CDNUTRIENTE AS CDNUTRIENTE3,
                NUT4.NMREDUZIDO AS NMREDUZIDO4,
                NUT4.SGUNIDADE  AS SG4,
                NUT4.CDNUTRIENTE AS CDNUTRIENTE4
            FROM
                PARAMPLANEJA
            LEFT JOIN NUTRIENTE NUT1 ON NUT1.CDNUTRIENTE = PARAMPLANEJA.CDNUTRRELAT1
            LEFT JOIN NUTRIENTE NUT2 ON NUT2.CDNUTRIENTE = PARAMPLANEJA.CDNUTRRELAT2
            LEFT JOIN NUTRIENTE NUT3 ON NUT3.CDNUTRIENTE = PARAMPLANEJA.CDNUTRRELAT3
            LEFT JOIN NUTRIENTE NUT4 ON NUT4.CDNUTRIENTE = PARAMPLANEJA.CDNUTRRELAT4
            WHERE PARAMPLANEJA.CDFILIAL = :P_CDFILCENTPME
            AND PARAMPLANEJA.NRORG = :P_NRORG";
      $params = array(
          'P_CDFILCENTPME' => $cdfilial,
          'P_NRORG' => $this->environment->getNrOrgTrab()
      );

      try {
          return $this->connection->fetchAll($sql, $params);
      } catch (\Exception $e) {
          throw $e;
      }
  }

  public function callProcedureNutrCard($central, $cdfilial, $cdservfili, $nutrientes, $idndpcal, $dtIni, $dtFin, $cdtipocard) {
		$nrorg = $this->environment->getNrOrgTrab();

		$nrSeqNutr = $this->newCode->getCodeByTableName('CALCNUTR'.$cdfilial, $nrorg);
		$procedureCons = new StoredProcedure($this->connection, 'NUTRIENTE_DO_CARDAPIO_PME');

		$procedureCons->addParam(new Param('P_CDFILCENTPME', Param::PARAM_INPUT, $central, 			Param::PARAM_TYPE_STR));
		$procedureCons->addParam(new Param('P_CDFILIAL',     Param::PARAM_INPUT, $cdfilial,     Param::PARAM_TYPE_STR));
		$procedureCons->addParam(new Param('P_CDSERVFILI',   Param::PARAM_INPUT, $cdservfili,   Param::PARAM_TYPE_STR));
		$procedureCons->addParam(new Param('P_NUTRIENTES',   Param::PARAM_INPUT, $nutrientes,   Param::PARAM_TYPE_STR));
		$procedureCons->addParam(new Param('P_NRSEQUNUTR',   Param::PARAM_INPUT, $nrSeqNutr,    Param::PARAM_TYPE_STR));
		$procedureCons->addParam(new Param('P_IDNDPCAL',     Param::PARAM_INPUT, $idndpcal,    	Param::PARAM_TYPE_STR));
		$procedureCons->addParam(new Param('P_DTINICIAL',    Param::PARAM_INPUT, $dtIni,        Param::PARAM_TYPE_STR));
		$procedureCons->addParam(new Param('P_DTFINAL',      Param::PARAM_INPUT, $dtFin,        Param::PARAM_TYPE_STR));
		$procedureCons->addParam(new Param('P_CDTIPOCARD',   Param::PARAM_INPUT, $cdtipocard,   Param::PARAM_TYPE_STR));
		$procedureCons->execute();
		
		return $nrSeqNutr;
  }
  
  public function consumoPlanPMe($cdfilcentpme, $cdtipocard, $filial, $servico, $initialDate, $finalDate, $idarquivo = 'T', $idprodutos = 'C',
                                 $idsepprato = 'P', $acrescimo = 0, $idarredond = 'N', $cdprodIni = '0', $cdprodFin = 'zzzzzzzzzz') {

      $consplan = $this->newCode->getCodeByTableName('CONSPLAN'.$cdfilcentpme, $this->environment->getNrOrgTrab());
      
      if(empty($cdtipocard)) $cdtipocard = null;
      else $cdtipocard = is_array($cdtipocard) ? join("_", $cdtipocard) : $cdtipocard;

      if(empty($filial)) $filial = null;
      else $filial = is_array($filial) ? join("_", $filial) : $filial;

      if(empty($servico)) $servico = null;
      else $servico = is_array($servico) ? join("_", $servico) : $servico;

		  $procedureCons = new StoredProcedure($this->connection, 'CONSUMO_PLANEJADO_PME');
    	
		  $procedureCons->addParam(new Param('P_CDFILCENTPME',  Param::PARAM_INPUT, $cdfilcentpme,   Param::PARAM_TYPE_STR));
    	$procedureCons->addParam(new Param('P_CDTIPOCARD',    Param::PARAM_INPUT, $cdtipocard,   Param::PARAM_TYPE_STR));
		  $procedureCons->addParam(new Param('P_CDFILIAL',      Param::PARAM_INPUT, $filial,   Param::PARAM_TYPE_STR));
    	$procedureCons->addParam(new Param('P_CDSERVFILI',    Param::PARAM_INPUT, $servico,   Param::PARAM_TYPE_STR));
    	$procedureCons->addParam(new Param('P_DTINICIAL',     Param::PARAM_INPUT, $initialDate, Param::PARAM_TYPE_STR));
    	$procedureCons->addParam(new Param('P_DTFINAL',       Param::PARAM_INPUT, $finalDate,   Param::PARAM_TYPE_STR));
    	$procedureCons->addParam(new Param('P_IDARQUIVO',     Param::PARAM_INPUT, $idarquivo,  Param::PARAM_TYPE_STR));
    	$procedureCons->addParam(new Param('P_IDPRODUTOS',    Param::PARAM_INPUT, $idprodutos,  Param::PARAM_TYPE_STR));
    	$procedureCons->addParam(new Param('P_IDSEPPRATO',    Param::PARAM_INPUT, $idsepprato,  Param::PARAM_TYPE_STR));
    	$procedureCons->addParam(new Param('P_PEACRESCIMO',   Param::PARAM_INPUT, $acrescimo,  Param::PARAM_TYPE_STR));
    	$procedureCons->addParam(new Param('P_IDARREDOND',    Param::PARAM_INPUT, $idarredond,  Param::PARAM_TYPE_STR));
    	$procedureCons->addParam(new Param('P_NRCONSPLAN',    Param::PARAM_INPUT, $consplan,  Param::PARAM_TYPE_STR));
    	$procedureCons->addParam(new Param('P_CDPRODUTOINI',  Param::PARAM_INPUT, $cdprodIni,  Param::PARAM_TYPE_STR));
    	$procedureCons->addParam(new Param('P_CDPRODUTOFIN',  Param::PARAM_INPUT, $cdprodFin,  Param::PARAM_TYPE_STR));
      $procedureCons->addParam(new Param('P_CDOPERADOR',    Param::PARAM_INPUT, $this->environment->getCdOperador(),  Param::PARAM_TYPE_STR));
      $procedureCons->execute();
    	
    	return $consplan;
    }
  
}